import os

from .io import set_json_value, load_json_value
from .time import validate_time

GHAGENT_HOME = os.path.expanduser("~/.ghagent")
GHARCHIVE_DIR = os.path.join(GHAGENT_HOME, "gharchive")
CONFIG_FILE_PATH = os.path.join(GHAGENT_HOME, "config.json")
LANCEDB_DIR = GHAGENT_HOME
EMBEDDING_MODEL_NAME = "all-MiniLM-L6-v2"

class ConfigValidationError(Exception):
    def __init__(self, message: str, status_code: int):
        """
        Error code:
          1001: config file path not existed
          1002: missing timewindow
          1003: missing GitHub API Token
        """
        super().__init__(message)
        self.status_code = status_code

def get_downloaded_gharchive_dates():
    start_date = None
    end_date = None
    files = os.listdir(GHARCHIVE_DIR)
    files.sort()
    if len(files) == 0:
        return start_date, end_date

def set_time_window(start_date: str, end_date: str):
    # sanity check
    if not validate_time(start_date):
        raise ValueError(f"Invalid start date: {start_date}")
    if not validate_time(end_date):
        raise ValueError(f"Invalid end date: {end_date}")
    
    set_json_value(CONFIG_FILE_PATH, "start_date", start_date)
    set_json_value(CONFIG_FILE_PATH, "end_date", end_date)

def set_github_token(token: str):
    set_json_value(CONFIG_FILE_PATH, "github_token", token)

def config_validation_check():
    """
    Various configuration setup and setting validation
    """
    # check config file exists
    if not os.path.exists(CONFIG_FILE_PATH):
        raise ConfigValidationError("Config file not found", 1001)
    
    # check time window is set
    if not load_json_value(CONFIG_FILE_PATH, "start_date", "") or not load_json_value(CONFIG_FILE_PATH, "end_date", ""):
        raise ConfigValidationError("Time window is not set", 1002)
    
    # check GitHub token is set
    if not load_json_value(CONFIG_FILE_PATH, "github_token", ""):
        raise ConfigValidationError("Github Access Token is not set", 1003)



import duckdb
import lancedb
import shutil
import pandas as pd
import pyarrow as pa
import requests
from sentence_transformers import SentenceTransformer
from pathlib import Path
from tqdm import tqdm

from ghagent.util.io import load_json_value
from ghagent.util.config import CONFIG_FILE_PATH, LANCEDB_DIR, ConfigValidationError

REPO_DB_DIR = Path.joinpath(Path.home(), "data/github_archive_analytics/vectordb/repo_info")
REPO_FILE = Path.joinpath(Path.home(), "data/github_archive_analytics/updated_expanded_repo_info.parquet")
BATCH_SIZE = 128
MICRO_BATCH_SIZE = 128 # used for GPU processing

MODEL_NAME="all-MiniLM-L6-v2"
MODEL = SentenceTransformer(MODEL_NAME)
EMBEDDING_SIZE = 384

def fetch_related_repo_info(repo: str, target_start_date: str, target_end_date: str):
    # get all related repos' urls from table
    db = lancedb.connect(LANCEDB_DIR)
    related_repos = db.open_table(f"{repo.replace('/', '-')}_{target_start_date}_{target_end_date}_related_repos").to_arrow()
    get_all_repos_urls_query = "SELECT DISTINCT related_repo_url FROM related_repos;"
    repo_urls = duckdb.sql(get_all_repos_urls_query).fetchall()
    
    # only request to urls that are not in `repo_info` table
    try:
        repo_info_tbl = db.open_table("repo_info")
        repo_info_tbl_arrow = repo_info_tbl.to_arrow()
        # NOTE: use single quote: https://stackoverflow.com/a/75671555
        repo_urls = [repo_url for repo_url in repo_urls if int(duckdb.sql(f"""SELECT COUNT(*) FROM repo_info_tbl_arrow WHERE repo_url = '{repo_url[0]}';""").fetchone()[0]) == 0]
    except FileNotFoundError as e:
        # since there is no `repo_info` table, all urls should be requested
        pass
    
    # request to urls to get repo's info
    access_token = load_json_value(CONFIG_FILE_PATH, "github_token", "")
    if not access_token:
        raise ConfigValidationError("Github Access Token is not set", 1003)
    headers = {
        "Authorization": f"token {access_token}",
    }
    repo_infos = []

    pbar = tqdm(repo_urls)
    for repo_url in pbar:
        pbar.set_description(f"Fetching {repo_url[0]}")

        try:
            response = requests.get(repo_url[0], headers=headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            # NOTE: should handle the exception properly
            print(f"Failed to request to GitHub API, reason: {response.text}")
            continue
        
        repo_info = response.json()
        repo_infos.append({
            "id": repo_info["id"],
            "description": repo_info["description"],
            "name": repo_info["name"],
            "full_name": repo_info["full_name"],
            "repo_url": repo_url[0],
        })
    
    # calculate the embeddings
    for batch in tqdm(range(len(repo_infos) // MICRO_BATCH_SIZE + 1)):
        start = batch * MICRO_BATCH_SIZE
        end = (batch + 1) * MICRO_BATCH_SIZE
        if start >= len(repo_infos):
            break
        if end > len(repo_infos):
            end = len(repo_infos)

        description_embeddings = MODEL.encode([repo_info["description"] for repo_info in repo_infos[start:end]], batch_size=MICRO_BATCH_SIZE)
        description_embeddings = [embedding.tolist() for embedding in description_embeddings]
        name_embeddings = MODEL.encode([repo_info["name"] for repo_info in repo_infos[start:end]], batch_size=MICRO_BATCH_SIZE)
        name_embeddings = [embedding.tolist() for embedding in name_embeddings]
        
        for i in range(start, end):
            repo_infos[i]["description_embedding"] = description_embeddings[i - start]
            repo_infos[i]["name_embedding"] = name_embeddings[i - start]

    # save the repo infos to `repo_info` table
    try:
        repo_info_tbl = db.open_table("repo_info")
        repo_info_tbl_arrow = repo_info_tbl.to_arrow()
        for repo_info in tqdm(repo_infos):
            is_exist = int(duckdb.sql(f'SELECT COUNT(*) FROM repo_info_tbl_arrow WHERE id = {repo_info["id"]};').fetchone()[0]) > 0
            if is_exist:
                # workaroud: duckdb's update operation has unresolved error
                # repo_info_tbl.update(where=f'id = {repo_info["id"]}', values=repo_info)
                repo_info_tbl.delete(where=f'id = {repo_info["id"]}')

            repo_info_tbl.add([repo_info])
    except FileNotFoundError as e:
        # create new table
        schema = pa.schema([
            pa.field("id", pa.int64()),
            pa.field("description", pa.utf8()),
            pa.field("name", pa.utf8()),
            pa.field("full_name", pa.utf8()),
            pa.field("repo_url", pa.utf8()),
            pa.field("description_embedding", pa.list_(pa.float32(), EMBEDDING_SIZE)),
            pa.field("name_embedding", pa.list_(pa.float32(), EMBEDDING_SIZE)),
        ])
        
        if len(repo_infos) == 0:
            # create empty table
            db.create_table("repo_info", schema=schema)
        else:
            db.create_table("repo_info", repo_infos, schema=schema)

def create_repo_info_table():
    db = lancedb.connect(REPO_DB_DIR)
    con = duckdb.connect()

    schema = pa.schema([
        pa.field("id", pa.int64()),
        pa.field("description", pa.utf8()),
        pa.field("name", pa.utf8()),
        pa.field("full_name", pa.utf8()),
        pa.field("description_embedding", pa.list_(pa.float32(), EMBEDDING_SIZE)),
        pa.field("name_embedding", pa.list_(pa.float32(), EMBEDDING_SIZE))
    ])

    repo_count = con.execute(f"select count(*) from '{REPO_FILE}';").fetchone()[0]
    
    def make_batches():
        query = con.execute(f"""select id, name, full_name, description from '{REPO_FILE}';""")
        for batch in tqdm(query.fetch_record_batch(BATCH_SIZE), total=(repo_count // BATCH_SIZE)):
            description_embeddings = MODEL.encode(batch["description"].to_pylist(), batch_size=MICRO_BATCH_SIZE)
            description_embeddings = [embedding.tolist() for embedding in description_embeddings]
            name_embeddings = MODEL.encode(batch["name"].to_pylist(), batch_size=MICRO_BATCH_SIZE)
            name_embeddings = [embedding.tolist() for embedding in name_embeddings]
            yield pa.RecordBatch.from_arrays(
                [
                    pa.array(batch["id"], pa.int64()),
                    pa.array(batch["description"]),
                    pa.array(batch["name"]),
                    pa.array(batch["full_name"]),
                    pa.array(description_embeddings, pa.list_(pa.float32(), EMBEDDING_SIZE)),
                    pa.array(name_embeddings, pa.list_(pa.float32(), EMBEDDING_SIZE))
                ],
                ["id", "description", "name", "full_name", "description_embedding", "name_embedding"],
            )

    shutil.rmtree(REPO_DB_DIR)
    db.create_table("repo_info", make_batches(), schema=schema)
    con.close()



#%%
import csv
import sys
import duckdb
from pathlib import Path
from tqdm import tqdm

DATA_DIR = Path.joinpath(Path.home(), "data/github_archive_analytics")
REPO_INFO_FILE = Path.joinpath(DATA_DIR, "repo_info.csv")
WATCH_EVENTS_FILE = Path.joinpath(DATA_DIR, "watch_events.csv")
MISSING_REPOS_FILE = Path.joinpath(DATA_DIR, "missing_repos.csv")

def extract_missing_repos():
    con = duckdb.connect()
    repos = con.execute(f"SELECT DISTINCT repo_full_name, repo_url from \"{WATCH_EVENTS_FILE}\";").fetchall()
    fetched_repos = con.execute(f"SELECT full_name from \"{REPO_INFO_FILE}\";").fetchall()
    fetched_repo_names = set([repo[0] for repo in fetched_repos])
    missed_repos = []
    for repo in tqdm(repos):
        if repo[0] not in fetched_repo_names:
            missed_repos.append(repo)
    with open(MISSING_REPOS_FILE, "w+") as f:
        writer = csv.writer(f)
        writer.writerow(["repo_full_name", "repo_url"])
        writer.writerows(missed_repos)
    con.close()

extract_missing_repos()



import modal
import datetime
import requests
import shutil
import boto3
import gzip
import duckdb
from modal import Image
from tqdm import tqdm

stub = modal.Stub("extract_watch_events")
image = Image.debian_slim().apt_install("wget").pip_install("pandas", "numpy", "requests", "tqdm", "boto3", "duckdb")
archive_volume = modal.Volume.new()
archive_path = "/vol/github-archives"

@stub.function(volumes={archive_path: archive_volume}, image=image)
def extract_watch_events(start_date, end_date):
    s3 = boto3.client(
        service_name ="s3",
        endpoint_url = 'https://a199e11f0d1168b4a64525b6ca1ee728.r2.cloudflarestorage.com',
        aws_access_key_id = '3a2f8c198906f447a7a059179280d4ff',
        aws_secret_access_key = 'f5d96367db6d13b4eccb943d8614ca209d74cfa946a19ccd91dfebf2449dfdeb',
        region_name="auto", # Must be one of: wnam, enam, weur, eeur, apac, auto
    )
    con = duckdb.connect()
    all_events = []
    # create list of date strings from start_date to end_date
    date_list = [start_date + datetime.timedelta(days=x) for x in range((end_date - start_date).days + 1)]
    date_strings = [date.strftime("%Y-%m-%d") for date in date_list]
    for date_string in tqdm(date_strings):
        for i in range(0, 24):
            # download the file form gharchive
            url = f"https://data.gharchive.org/{date_string}-{i}.json.gz"
            filename = f"{archive_path}/{date_string}-{i}.json.gz"
            jsonfile = f"{archive_path}/{date_string}-{i}.json"
            r = requests.get(url, stream=True)
            if r.status_code == 200:
                with open(filename, 'wb') as f:
                    r.raw.decode_content = True
                    shutil.copyfileobj(r.raw, f)
            # extract the .gz file
            with gzip.open(filename, 'rb') as f_in:
                with open(jsonfile, 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)
            # extract the watch events
            events = con.execute(f"SELECT actor.login, actor.url, repo.name, repo.url, created_at from read_json_auto('{jsonfile}', ignore_errors=true) where type='WatchEvent';").fetchall()
            all_events.extend(events)
    con.close()
    return True

@stub.local_entrypoint()
def main():
    start_date = datetime.date(2022, 1, 1)
    end_date = datetime.date(2023, 9, 26)
    extract_watch_events.remote(start_date, end_date)
    # dates = [
    #     (datetime.date(2022, 1, 1), datetime.date(2022, 12, 31)),
    #     (datetime.date(2023, 1, 1), datetime.date(2023, 9, 26))
    # ]
    # list(download_github_archives.starmap(dates))



#%%
from typing import List
import duckdb
import lancedb
import pyarrow as pa
from sentence_transformers import SentenceTransformer
from pathlib import Path
from tqdm import tqdm

DATA_DIR = Path.joinpath(Path.home(), "data/github_archive_analytics")
CACHE_DB = Path.joinpath(DATA_DIR, "cache/cache.db")
# GITHUB_EVENTS_DIR = Path.joinpath(Path.home(), "nfs/datasets/github-events")
GITHUB_EVENTS_DIR = Path.joinpath(DATA_DIR, "github_events")
RELATED_USERS_DIR = Path.joinpath(DATA_DIR, "related_users")
RELATED_REPOS_DIR = Path.joinpath(DATA_DIR, "related_repos")
LANCEDB_DIR = Path.joinpath(DATA_DIR, "vectordb/github_analytics")
MODEL_NAME="all-MiniLM-L6-v2"
MODEL = SentenceTransformer(MODEL_NAME)
EMBEDDING_SIZE = 384

def get_all_event_dates() -> List[str]:
    dates = []
    for file in GITHUB_EVENTS_DIR.glob("*.parquet"):
        dates.append(file.name.split(".")[0])
    dates.sort()
    return dates

def extract_related_users(repo:str):
    dates = get_all_event_dates()
    db = lancedb.connect(LANCEDB_DIR)
    # step 1: get repo's all stargazers
    print("Getting repo's stargazers")
    stargazer_logins = set()
    for date in tqdm(dates):
        event_file = Path.joinpath(GITHUB_EVENTS_DIR, f"{date}.parquet")
        query = f"SELECT DISTINCT actor.login as login FROM '{event_file}' WHERE type = 'WatchEvent' AND repo.name = '{repo}';"
        stargazer_logins.update(duckdb.sql(query).fetchdf()["login"].tolist())
    stargazer_logins = pa.Table.from_arrays([pa.array(list(stargazer_logins), pa.utf8())], names=["login"])
    # step 2: get all the repos that the repo's stargazersers starred
    print("Getting stargazers' starred repos")
    related_repos_sub_tables = []
    for date in tqdm(dates):
        event_file = Path.joinpath(GITHUB_EVENTS_DIR, f"{date}.parquet")
        query = f"SELECT repo.name AS related_repo, actor.login AS common_stargazer, first(created_at) AS created_at FROM '{event_file}' WHERE actor.login IN (SELECT * FROM stargazer_logins) GROUP BY related_repo, common_stargazer;"
        sub_table = pa.Table.from_pandas(duckdb.sql(query).fetchdf())
        related_repos_sub_tables.append(sub_table)
    related_repos_table = pa.concat_tables(related_repos_sub_tables)
    related_repos_table = related_repos_table.group_by(["related_repo", "common_stargazer"]).aggregate([("created_at", "min")])
    related_repos_table_name = f"{repo.replace('/', '-')}_related_repos"
    db.drop_table(related_repos_table_name, ignore_missing=True)
    db.create_table(related_repos_table_name, related_repos_table)
    # step 3: get all the users who starred the repos
    print("Getting related repos' stargazers")
    related_repos = duckdb.sql("SELECT DISTINCT related_repo AS repo_name FROM related_repos_table;").fetch_arrow_table()
    related_users_sub_tables = []
    for date in tqdm(dates):
        event_file = Path.joinpath(GITHUB_EVENTS_DIR, f"{date}.parquet")
        query = f"SELECT repo.name AS related_repo, actor.login AS related_user, first(created_at) AS created_at FROM '{event_file}' WHERE repo.name IN (SELECT * FROM related_repos) GROUP BY related_repo, related_user;"
        sub_table = pa.Table.from_pandas(duckdb.sql(query).fetchdf())
        related_users_sub_tables.append(sub_table)
    related_users_table = pa.concat_tables(related_users_sub_tables)
    related_users_table = related_users_table.group_by(["related_repo", "related_user"]).aggregate([("created_at", "min")])
    related_users_table_name = f"{repo.replace('/', '-')}_related_users"
    db.drop_table(related_users_table_name, ignore_missing=True)
    db.create_table(related_users_table_name, related_users_table)

def query_related_users(repo: str, query: str):
    db = lancedb.connect(LANCEDB_DIR)
    repo_info = db.open_table("repo_info")
    if not query.strip():
        relevant_repos = repo_info.to_arrow()
    else:
        query_vector = MODEL.encode([query])[0]
        relevant_repos = repo_info.search(query_vector, "description_embedding").to_arrow()
    related_users = db.open_table(f"{repo.replace('/', '-')}_related_users").to_arrow()
    query = "SELECT related_repo, STRING_AGG(related_user), COUNT(related_user) FROM related_users WHERE related_repo IN (SELECT full_name FROM relevant_repos) GROUP BY related_repo;"
    all_relevant_related_repo_user = duckdb.sql(query).fetchall()
    return all_relevant_related_repo_user

extract_related_users("litanlitudan/skyagi")
# extract_related_users('unx21/violet')
# query_related_users('unx21/violet', 'hmm')
# repo = "unx21/violet"
# %%



from typing import Optional
import pandas as pd
from rich.table import Table
from textual.widgets import DataTable

# Reference: https://gist.github.com/neelabalan/33ab34cf65b43e305c3f12ec6db05938
def df_to_rich_table(
    pandas_dataframe: pd.DataFrame,
    rich_table: Table,
    show_index: bool = True,
    index_name: Optional[str] = None,
) -> Table:
    """Convert a pandas.DataFrame obj into a rich.Table obj.
    Args:
        pandas_dataframe (DataFrame): A Pandas DataFrame to be converted to a rich Table.
        rich_table (Table): A rich Table that should be populated by the DataFrame values.
        show_index (bool): Add a column with a row count to the table. Defaults to True.
        index_name (str, optional): The column name to give to the index column. Defaults to None, showing no value.
    Returns:
        Table: The rich Table instance passed, populated with the DataFrame values."""

    if show_index:
        index_name = str(index_name) if index_name else ""
        rich_table.add_column(index_name)

    for column in pandas_dataframe.columns:
        rich_table.add_column(str(column))

    for index, value_list in enumerate(pandas_dataframe.values.tolist()):
        row = [str(index)] if show_index else []
        row += [str(x) for x in value_list]
        rich_table.add_row(*row)

    return rich_table

def df_to_textual_table(
    pandas_dataframe: pd.DataFrame,
    textual_table: DataTable,
    show_index: bool = True,
    index_name: Optional[str] = None,
) -> DataTable:
    """Convert a pandas.DataFrame obj into a textual.widgets.DataTable obj.
    Args:
        pandas_dataframe (DataFrame): A Pandas DataFrame to be converted to a textual DataTable.
        textual_table (DataTable): A textual DataTable that should be populated by the DataFrame values.
        show_index (bool): Add a column with a row count to the table. Defaults to True.
        index_name (str, optional): The column name to give to the index column. Defaults to None, showing no value.
    Returns:
        DataTable: The textual DataTable instance passed, populated with the DataFrame values."""
    
    if show_index:
        index_name = str(index_name) if index_name else ""
        textual_table.add_column(index_name)

    for column in pandas_dataframe.columns:
        textual_table.add_column(str(column))

    for index, value_list in enumerate(pandas_dataframe.values.tolist()):
        row = [str(index)] if show_index else []
        row += [str(x) for x in value_list]
        textual_table.add_row(*row)

    return textual_table



from datetime import datetime


def validate_time(time_str: str):
    """Validate time string

    Args:
        time_str (str): Time string

    Returns:
        bool: True if valid, False otherwise
    """
    try:
        datetime.strptime(time_str, '%Y-%m-%d')
        return True
    except ValueError:
        return False



import duckdb
import lancedb
import numpy as np
import gradio as gr
import pyarrow as pa
from sentence_transformers import SentenceTransformer
from pathlib import Path
from typing import List
from tqdm import tqdm

DATA_DIR = Path.joinpath(Path.home(), "data/github_archive_analytics")
GITHUB_EVENTS_DIR = Path.joinpath(DATA_DIR, "github_events")
LANCEDB_DIR = Path.joinpath(DATA_DIR, "vectordb/github_analytics")

MODEL_NAME="all-MiniLM-L6-v2"
MODEL = SentenceTransformer(MODEL_NAME)
EMBEDDING_SIZE = 384

def get_all_event_dates() -> List[str]:
    dates = []
    for file in GITHUB_EVENTS_DIR.glob("*.parquet"):
        dates.append(file.name.split(".")[0])
    dates.sort()
    return dates

def get_processed_repos() -> List[str]:
    db = lancedb.connect(LANCEDB_DIR)
    tables = db.table_names()
    suffix = "_related_users"
    repos = [table[:-(len(suffix))] for table in tables if table.endswith(suffix)]
    return repos

def search_relevant_related_repos(repo: str, query: str):
    if not repo:
        return None, 0, None, 0
    db = lancedb.connect(LANCEDB_DIR)
    repo_info = db.open_table("repo_info")
    related_repos = db.open_table(f"{repo}_related_repos").to_arrow()
    related_users = db.open_table(f"{repo}_related_users").to_arrow()
    if not query.strip():
        repo_info_arrow = repo_info.to_arrow()
    else:
        query_vector = MODEL.encode([query.strip()])[0]
        repo_info_arrow = repo_info.search(query_vector, "description_embedding").to_arrow()
    # step 1: find relevant related repos
    relevant_related_repos = duckdb.sql("SELECT r1.related_repo[:50] AS related_repo, r2.description[:50] AS description, string_agg(r1.common_stargazer)[:20] AS common_stargazers, count(r1.common_stargazer) AS common_stargazer_cnt FROM related_repos r1 JOIN repo_info_arrow r2 ON r1.related_repo = r2.full_name GROUP BY related_repo, description ORDER BY common_stargazer_cnt DESC LIMIT 1024;").fetch_arrow_table()
    # step 2: find relevant related users
    relevant_related_users = duckdb.sql("SELECT r1.related_user[:20] AS related_user, string_agg(r1.related_repo)[:50] as common_repos, count(r1.related_repo) AS common_repo_cnt FROM related_users r1 JOIN repo_info_arrow r2 ON r1.related_repo = r2.full_name GROUP BY related_user ORDER BY common_repo_cnt DESC LIMIT 1024;").fetch_arrow_table()
    return relevant_related_repos.to_pandas(), relevant_related_repos.num_rows, relevant_related_users.to_pandas(), relevant_related_users.num_rows

def launch():
    with gr.Blocks() as demo:
        gr.Markdown("Github-Analytics 🚀🚀🚀")

        with gr.Row():
            repo = gr.Dropdown(get_processed_repos(), label="Select a Repo")
            query = gr.Textbox(label="Search Query", placeholder="Search for a repo")
        search_button = gr.Button("Search")

        with gr.Accordion("Related Repos"):
            with gr.Row():
                relevant_related_repos_cnt = gr.Number(0, label="Number of relevant-related repos")
                gr.Label("🔥🔥🔥")
            relevant_related_repos = gr.DataFrame(headers=["Repo Name", "Description", "Common Stargazers", "Common Stargazer Count"], wrap=True)

        relevant_related_users_cnt = gr.Number(0, label="Number of relevant-related users")
        relevant_related_users = gr.DataFrame(headers=["User Name", "Common Stargazer Count", "Created At"])
        search_button.click(search_relevant_related_repos, inputs=[repo, query], outputs=[relevant_related_repos, relevant_related_repos_cnt, relevant_related_users, relevant_related_users_cnt])

    demo.launch()

if __name__ == "__main__":
    launch()



# lep photon create -n github-analytics-data -m lepton_data.py
# lep photon push -n github-analytics-data
# lep deployment remove -n github-analytics-data
# lep photon run -n github-analytics-data -dn github-analytics-data --mount /github-analytics/data:/mnt/data

from leptonai.photon import Photon
import os
import requests
import shutil
import duckdb
import gzip
from pathlib import Path

class GithubAnalyticsData(Photon):
    requirement_dependency = ["duckdb", "lancedb"]

    def init(self):
        self.counter = 0
        self.data_dir = Path("/mnt/data")

    def _fetch_archive(self, date: str):
        # date: 2021-01-01
        urls = [f"https://data.gharchive.org/{date}-{hour}.json.gz" for hour in range(0, 24)]
        download_dir = Path.joinpath(self.data_dir, "download", date)
        archive_dir = Path.joinpath(self.data_dir, "github-archive")
        os.makedirs(download_dir, exist_ok=True)
        os.makedirs(archive_dir, exist_ok=True)
        # step 1: download the file form gharchive
        for url in urls:
            print(f"Downloading {url}")
            filename = Path.joinpath(download_dir, url.split("/")[-1])
            r = requests.get(url, stream=True)
            if r.status_code == 200:
                with open(filename, 'wb') as f:
                    r.raw.decode_content = True
                    shutil.copyfileobj(r.raw, f)
        # step 2: unzip the file to download_dir
        for file in download_dir.glob("*.gz"):
            output_file_path = Path.joinpath(download_dir, file.name[:-3])
            with gzip.open(file, 'rb') as f_in:
                with open(output_file_path, 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)
        # step 3: aggregate the file to archive_dir as a single date.parquet
        print(f"Aggregating json files to {date}.parquet")
        parquet_file = Path.joinpath(archive_dir, f"{date}.parquet")
        query = f"COPY (SELECT * FROM read_json_auto('{download_dir}/*.json', ignore_errors=true)) TO '{parquet_file}';"
        duckdb.sql(query)
        # step 4: delete the download_dir
        for file_path in download_dir.glob('*'):
            try:
                if file_path.is_file():
                    print(f"Deleting file {file_path}")
                    file_path.unlink()
            except Exception as e:
                print(f"Error deleting file: {e}")
        download_dir.rmdir()

    @Photon.handler()
    def fetch_archive(self, date: str) -> bool:
        self.add_background_task(self._fetch_archive, date)
        return True



#%%
import csv
import sys
import duckdb
from pathlib import Path
from tqdm import tqdm
import lancedb
import pyarrow as pa
from sentence_transformers import SentenceTransformer

from ghagent.util.config import EMBEDDING_MODEL_NAME, GHARCHIVE_DIR, LANCEDB_DIR
from ghagent.util.gharchive import gen_date_list

DATA_DIR = Path.joinpath(Path.home(), "data/github_archive_analytics")
ANALYTICS_DIR = Path.joinpath(DATA_DIR, "analytics")
USER_INFO_FILE = Path.joinpath(DATA_DIR, "user_info.csv")
WATCH_EVENTS_FILE = Path.joinpath(DATA_DIR, "watch_events.csv")
MISSING_USERS_FILE = Path.joinpath(DATA_DIR, "missing_users.csv")

def extract_related_repos(repo:str):
    con = duckdb.connect()
    query = f"""
SELECT DISTINCT r2.repo_full_name, r1.stargazer_login AS common_stargazer, r1.created_at as create_at
FROM '{WATCH_EVENTS_FILE}' r1
JOIN '{WATCH_EVENTS_FILE}' r2 ON r1.stargazer_login = r2.stargazer_login
WHERE r1.repo_full_name = '{repo}' AND r2.repo_full_name <> '{repo}';
""".strip()
    repo_stargazers = con.execute(query).fetchall()
    with open(f"{ANALYTICS_DIR}/related_repos_{repo.replace('/', '-')}.csv", "w+") as f:
        writer = csv.writer(f)
        writer.writerow(["repo_full_name", "common_stargazer", "created_at"])
        writer.writerows(repo_stargazers)
    con.close()

def extract_related_watch_events(repo:str):
    con = duckdb.connect()
    related_repos_file = f"{ANALYTICS_DIR}/related_repos_{repo.replace('/', '-')}.csv"
    related_watch_events_file = f"{ANALYTICS_DIR}/related_watch_events_{repo.replace('/', '-')}.csv"
    query = f"""
SELECT * FROM '{WATCH_EVENTS_FILE}'
WHERE repo_full_name in (SELECT repo_full_name FROM '{related_repos_file}');
""".strip()
    related_watch_events = con.execute(query).fetchall()
    with open(related_watch_events_file, "w+") as f:
        writer = csv.writer(f)
        writer.writerow(["stargazer_login", "stargazer_url", "repo_full_name", "repo_url", "created_at"])
        writer.writerows(related_watch_events)
    con.close()

repo = "litanlitudan/skyagi"
# extract_related_repos(repo)
# extract_related_watch_events(repo)


def get_related_repos(repo: str, target_start_date: str, target_end_date: str):
    dates = gen_date_list(target_start_date, target_end_date)
    db = lancedb.connect(LANCEDB_DIR)

    # step 1: get repo's all stargazers
    print("Getting repo's stargazers")
    stargazer_logins = set()
    for date in tqdm(dates):
        event_file = Path.joinpath(Path(GHARCHIVE_DIR), f"{date}.parquet")
        query = f"SELECT DISTINCT actor.login as login FROM '{event_file}' WHERE type = 'WatchEvent' AND repo.name = '{repo}';"
        stargazer_logins.update(duckdb.sql(query).fetchdf()["login"].tolist())
    stargazer_logins = pa.Table.from_arrays([pa.array(list(stargazer_logins), pa.utf8())], names=["login"])

    # step 2: get all the repos that the repo's stargazersers starred
    print("Getting stargazers' starred repos")
    related_repos_sub_tables = []
    for date in tqdm(dates):
        event_file = Path.joinpath(Path(GHARCHIVE_DIR), f"{date}.parquet")
        query = f"SELECT repo.name AS related_repo, repo.url as related_repo_url, actor.login AS common_stargazer, first(created_at) AS created_at FROM '{event_file}' WHERE actor.login IN (SELECT * FROM stargazer_logins) GROUP BY related_repo, related_repo_url, common_stargazer;"
        sub_table = pa.Table.from_pandas(duckdb.sql(query).fetchdf())
        related_repos_sub_tables.append(sub_table)
    related_repos_table = pa.concat_tables(related_repos_sub_tables)
    related_repos_table = related_repos_table.group_by(["related_repo", "related_repo_url", "common_stargazer"]).aggregate([("created_at", "min")])
    # NOTE: table name doesn't show embedding model yet
    related_repos_table_name = f"{repo.replace('/', '-')}_{target_start_date}_{target_end_date}_related_repos"
    db.drop_table(related_repos_table_name, ignore_missing=True)
    db.create_table(related_repos_table_name, related_repos_table, schema=pa.schema([
        pa.field("related_repo", pa.utf8()),
        pa.field("related_repo_url", pa.utf8()),
        pa.field("common_stargazer", pa.utf8()),
        pa.field("created_at_min", pa.timestamp('us')),
    ]))

def find_relevant_related_repos(repo: str, target_start_date: str, target_end_date: str, query: str):
    db = lancedb.connect(LANCEDB_DIR)
    repo_info = db.open_table("repo_info")
    related_repos = db.open_table(f"{repo.replace('/', '-')}_{target_start_date}_{target_end_date}_related_repos").to_arrow()
    if not query.strip():
        repo_info_arrow = repo_info.to_arrow()
    else:
        query_vector = SentenceTransformer(EMBEDDING_MODEL_NAME).encode([query.strip()])[0]
        repo_info_arrow = repo_info.search(query_vector, "description_embedding").to_arrow()
        
    relevant_related_repos = duckdb.sql("SELECT r1.related_repo[:50] AS related_repo, r2.description[:50] AS description, string_agg(r1.common_stargazer)[:20] AS common_stargazers, count(r1.common_stargazer) AS common_stargazer_cnt FROM related_repos r1 JOIN repo_info_arrow r2 ON r1.related_repo = r2.full_name GROUP BY related_repo, description ORDER BY common_stargazer_cnt DESC LIMIT 1024;").fetch_arrow_table()
    
    return relevant_related_repos.to_pandas()



from pathlib import Path
import duckdb
import lancedb
from tqdm import tqdm
import pyarrow as pa
from sentence_transformers import SentenceTransformer

from ghagent.util.gharchive import gen_date_list
from ghagent.util.config import EMBEDDING_MODEL_NAME, GHARCHIVE_DIR, LANCEDB_DIR

def get_related_stargazers_from_related_repos(repo: str, target_start_date: str, target_end_date: str):
    dates = gen_date_list(target_start_date, target_end_date)
    db = lancedb.connect(LANCEDB_DIR)
    
    # get all the users who starred the related repos
    print("Getting related repos' stargazers")
    # NOTE: table name doesn't show embedding model yet
    related_repos_table_name = f"{repo.replace('/', '-')}_{target_start_date}_{target_end_date}_related_repos"
    related_repos_table = db.open_table(related_repos_table_name).to_arrow()
    related_repos = duckdb.sql(f"SELECT DISTINCT related_repo AS repo_name FROM related_repos_table;").fetch_arrow_table()
    related_users_sub_tables = []

    for date in tqdm(dates):
        event_file = Path.joinpath(Path(GHARCHIVE_DIR), f"{date}.parquet")
        query = f"SELECT repo.name AS related_repo, repo.url as related_repo_url, actor.login AS related_user, first(created_at) AS created_at FROM '{event_file}' WHERE repo.name IN (SELECT * FROM related_repos) GROUP BY related_repo, related_repo_url, related_user;"
        sub_table = pa.Table.from_pandas(duckdb.sql(query).fetchdf())
        related_users_sub_tables.append(sub_table)
    related_users_table = pa.concat_tables(related_users_sub_tables)
    related_users_table = related_users_table.group_by(["related_repo", "related_repo_url", "related_user"]).aggregate([("created_at", "min")])
    # NOTE: table name doesn't show embedding model yet
    related_users_table_name = f"{repo.replace('/', '-')}_{target_start_date}_{target_end_date}_related_users"
    db.drop_table(related_users_table_name, ignore_missing=True)
    db.create_table(related_users_table_name, related_users_table, schema=pa.schema([
        pa.field("related_repo", pa.utf8()),
        pa.field("related_repo_url", pa.utf8()),
        pa.field("related_user", pa.utf8()),
        pa.field("created_at_min", pa.timestamp('us')),
    ]))

def find_relevant_potential_stargazers(repo: str, target_start_date: str, target_end_date: str, query: str):
    db = lancedb.connect(LANCEDB_DIR)
    repo_info = db.open_table("repo_info")
    related_users = db.open_table(f"{repo.replace('/', '-')}_{target_start_date}_{target_end_date}_related_users").to_arrow()
    if not query.strip():
        repo_info_arrow = repo_info.to_arrow()
    else:
        query_vector = SentenceTransformer(EMBEDDING_MODEL_NAME).encode([query.strip()])[0]
        repo_info_arrow = repo_info.search(query_vector, "description_embedding").to_arrow()
        
    relevant_potential_stargazers = duckdb.sql("SELECT r1.related_user[:20] AS related_user, string_agg(r1.related_repo)[:50] as common_repos, count(r1.related_repo) AS common_repo_cnt FROM related_users r1 JOIN repo_info_arrow r2 ON r1.related_repo = r2.full_name GROUP BY related_user ORDER BY common_repo_cnt DESC LIMIT 1024;").fetch_arrow_table()
    
    return relevant_potential_stargazers.to_pandas()



import os
from datetime import datetime, timedelta
from leptonai.client import Client, current
from dotenv import load_dotenv
from tqdm import tqdm

load_dotenv()

today = datetime.today()
client = Client("w3mgia27", "github-analytics-data", token=os.getenv("LEPTON_API_TOKEN"))

for i in tqdm(range(1000)):
    day = today - timedelta(days=i)
    date = day.strftime('%Y-%m-%d')
    result = client.fetch_archive(date=date)
    print(result)



import os
from ghagent.tui import GHAgentApp
from ghagent.util.conversion import df_to_rich_table
import typer
import json
from rich.prompt import Prompt
from rich.console import Console
from rich.table import Table
from rich import box

from .util.gharchive import fetch_events, get_event_dates_to_download
from .util.io import load_json_value
from .util.config import CONFIG_FILE_PATH, GHAGENT_HOME, GHARCHIVE_DIR, set_github_token, set_time_window, ConfigValidationError, config_validation_check
from .pipeline.extract_related_repos import find_relevant_related_repos, get_related_repos
from .pipeline.extract_related_stargazers import find_relevant_potential_stargazers, get_related_stargazers_from_related_repos
from .pipeline.encode_related_repos import fetch_related_repo_info


cli = typer.Typer()
console = Console()

#######################################################################################
# Config CLI

config_cli = typer.Typer()

@config_cli.command("timewindow")
def config_timewindow():
    """
    Configure the time window for the app
    """
    console.print("Configuring time window...")
    start_date = Prompt.ask("Please input the start date of the time window", default="2023-05-01")
    end_date = Prompt.ask("Please input the end date of the time window", default="2023-05-01")
    
    try:
        set_time_window(start_date=start_date, end_date=end_date)
    except FileNotFoundError as e:
        console.print("Didn't find config file, run `gha init` to initialize.")
    
@config_cli.command("token")
def config_token():
    """
    Configure the github API token
    """
    console.print("Configuring Github token...")
    token = Prompt.ask("Please enter your Github API token")

    try:
        set_github_token(token)
    except FileNotFoundError as e:
        console.print("Didn't find config file, run `gha init` to initialize.")
    
@config_cli.callback(no_args_is_help=True)
def main(ctx: typer.Context):
    """
    Configuration
    """

cli.add_typer(config_cli, name="config")

#######################################################################################
# Main CLI

@cli.callback(no_args_is_help=True)
def main(ctx: typer.Context):
    """
    """
    pass


@cli.command()
def run(repo: str, output_format: str = "textual"):
    """
    Run the GHAgent for a given repository
    """
    print(f"Running GHAgent for {repo}...")
    
    # sanity check
    try:
        config_validation_check()
    except ConfigValidationError as e:
        if e.status_code == 1001:
            console.print("[red]Didn't find config file, please run `gha init` to initialize.")
        elif e.status_code == 1002:
            console.print("[red]Didn't have explicit start or end date set, please run `gha config timewindow` to specify properly.")
        elif e.status_code == 1003:
            console.print("[red]Didn't find GitHub API Access Token, please run `gha config token` to specify properly.")
        return

    # Fetch Events
    target_start_date = load_json_value(CONFIG_FILE_PATH, "start_date", "")
    target_end_date = load_json_value(CONFIG_FILE_PATH, "end_date", "")
    console.print(f"Configed time window: from {target_start_date} to {target_end_date}")
    if len(target_start_date) == 0 or len(target_end_date) == 0:
        console.print(f"Start date or end date is empty, please specify them by running `gha config timewindow` properly.")
    confirm = Prompt.ask("Please confirm the time window, do you want to continue? (y/n)", choices=["y", "n"], default="y")
    if confirm == "n":
        return
    
    event_dates_to_download = get_event_dates_to_download(target_start_date=target_start_date, target_end_date=target_end_date, output_dir=GHARCHIVE_DIR)
    if len(event_dates_to_download) > 0:
        console.print(f"{len(event_dates_to_download)} dates' data are not found locally, which need to download remotely: {event_dates_to_download}")

        ## Prompt to ask if the user wants to fetch the data
        confirm = Prompt.ask("Do you want to download the data? (y/n)", choices=["y", "n"], default="y")
        if confirm == "y":
            fetch_events(event_dates_to_download, output_dir=GHARCHIVE_DIR)
        else:
            return
    
    # Create the related repos table in lancedb
    get_related_repos(repo=repo, target_start_date=target_start_date, target_end_date=target_end_date)
    
    # Create the related stargazers table in lancedb
    get_related_stargazers_from_related_repos(repo=repo, target_start_date=target_start_date, target_end_date=target_end_date)
    
    # Calculate the embeddings of repo's name and description, and save them to `repo_info` table in lancedb
    fetch_related_repo_info(repo=repo, target_start_date=target_start_date, target_end_date=target_end_date)
    
    # Get the query from the user to filter
    query = Prompt.ask("Please input your query to filter relevant related repos and potential stargazers, leave it empty if you don't have any query", default="")
    
    # Find relevant related repos
    relevant_related_repos = find_relevant_related_repos(repo=repo, target_start_date=target_start_date, target_end_date=target_end_date, query=query)
    
    # Find relevant potential stargazers
    relevant_potential_stargazers = find_relevant_potential_stargazers(repo=repo, target_start_date=target_start_date, target_end_date=target_end_date, query=query)
    
    # Show the final result to the user
    if output_format == "textual":
        app = GHAgentApp(relevant_related_repos=relevant_related_repos, potential_stargazers=relevant_potential_stargazers)
        app.run()
    elif output_format == "rich":
        console.print("Related Repos", style="bold cyan")
        relevant_related_repos_table = Table(show_header=True, header_style="bold magenta")
        relevant_related_repos_table = df_to_rich_table(relevant_related_repos, relevant_related_repos_table)
        relevant_related_repos_table.row_styles = ["none", "dim"]
        relevant_related_repos_table.box = box.SIMPLE_HEAD
        console.print(relevant_related_repos_table)
        
        console.print("Potential Stargazers", style="bold cyan")
        relevant_potential_stargazers_table = Table(show_header=True, header_style="bold magenta")
        relevant_potential_stargazers_table = df_to_rich_table(relevant_potential_stargazers, relevant_potential_stargazers_table)
        relevant_potential_stargazers_table.row_styles = ["none", "dim"]
        relevant_potential_stargazers_table.box = box.SIMPLE_HEAD
        console.print(relevant_potential_stargazers_table)

@cli.command()
def init():
    """
    Initialize the GHAgent environment at ~/.ghagent
    ~/.ghagent/gharchive/: 2020-01-01.parquet, 2020-01-02.parquet, ...
    """
    print("Initializing GHAgent environment...")
    # Create ~/.ghagent if not exists
    if not os.path.exists(GHAGENT_HOME):
        os.makedirs(GHAGENT_HOME)
    # Create ~/.ghagent/config.json if not exists
    if not os.path.exists(CONFIG_FILE_PATH):
        with open(CONFIG_FILE_PATH, "w") as f:
            json.dump({}, f)



import os
import gzip
from typing import List, Union
import requests
import duckdb
import shutil
from datetime import datetime, timedelta
from pathlib import Path
from tqdm import tqdm

from ghagent.util.config import GHARCHIVE_DIR


def download_github_archive(date: str, output_dir: Union[str, Path]):
    """
    Download the github archive for a given date YYYY-MM-DD
    GHArchive website: https://www.gharchive.org/
    """
    if isinstance(output_dir, str):
        output_dir = Path(output_dir)
    else:
        output_dir = output_dir
    urls = [f"https://data.gharchive.org/{date}-{hour}.json.gz" for hour in range(0, 24)]
    # NOTE: for dev/test purpose, only download/handle the first hour's data
    # urls = [f"https://data.gharchive.org/{date}-{hour}.json.gz" for hour in range(0, 1)]
    download_dir = Path.joinpath(output_dir, "download", date)
    os.makedirs(download_dir, exist_ok=True)
    os.makedirs(output_dir, exist_ok=True)
    # step 1: download the file form gharchive
    for url in urls:
        print(f"Downloading {url}")
        filename = Path.joinpath(download_dir, url.split("/")[-1])
        if filename.exists():
            print(f"File {filename} already exists, skipping download")
            continue
        r = requests.get(url, stream=True)
        if r.status_code == 200:
            with open(filename, 'wb') as f:
                r.raw.decode_content = True
                shutil.copyfileobj(r.raw, f)
    # step 2: unzip the file to download_dir
    for file in download_dir.glob("*.gz"):
        print(f"Extracting file {file.name}")
        output_file_path = Path.joinpath(download_dir, file.name[:-3])
        if output_file_path.exists():
            print(f"File {output_file_path} already exists, skipping extraction")
            continue
        with gzip.open(file, 'rb') as f_in:
            with open(output_file_path, 'wb') as f_out:
                shutil.copyfileobj(f_in, f_out)
    # step 3: aggregate the file to archive_dir as a single date.parquet
    print(f"Aggregating json files to {date}.parquet")
    parquet_file = Path.joinpath(output_dir, f"{date}.parquet")
    query = f"COPY (SELECT * FROM read_json_auto('{download_dir}/*.json', ignore_errors=true)) TO '{parquet_file}';"
    try:
        duckdb.sql(query)
    except Exception as e:
        print(f"Error aggregating json files: {e}")
        print(f"Query: {query}")
        # delete file
        parquet_file.unlink()
        raise(e)
    # step 4: delete the download_dir
    for file_path in download_dir.glob('*'):
        try:
            if file_path.is_file():
                print(f"Deleting file {file_path}")
                file_path.unlink()
        except Exception as e:
            print(f"Error deleting file: {e}")
    download_dir.rmdir()


def gen_date_list(start_date: str, end_date: str):
    """
    Generate a list of dates between start_date and end_date in the format YYYY-MM-DD
    """
    start_date = datetime.strptime(start_date, '%Y-%m-%d')
    end_date = datetime.strptime(end_date, '%Y-%m-%d')
    date_list = []

    while start_date <= end_date:
        date_list.append(end_date.strftime('%Y-%m-%d'))
        end_date -= timedelta(days=1)
    return date_list

def get_all_event_dates(dir_path: Union[str, Path] = GHARCHIVE_DIR) -> List[str]:
    if isinstance(dir_path, str):
        dir_path = Path(dir_path)
    else:
        dir_path = dir_path
    dates = []
    for file in dir_path.glob("*.parquet"):
        dates.append(file.name.split(".")[0])
    dates.sort()
    return dates

def fetch_events(target_date_list: List[str], output_dir: Union[str, Path] = GHARCHIVE_DIR):
    """Fetch the repos for a given list of dates"""
    for target_date in tqdm(target_date_list):
        download_github_archive(target_date, output_dir)
        
def get_event_dates_to_download(target_start_date: str, target_end_date: str, output_dir: Union[str, Path] = GHARCHIVE_DIR):
    target_date_list = gen_date_list(target_start_date, target_end_date)
    existing_date_set = set(get_all_event_dates(output_dir))
    return [target_date for target_date in target_date_list if target_date not in existing_date_set]



from leptonai.photon import Photon

class GithubAnalyticsAPP(Photon):
    # The init method implements any custom initialization logic we need.
    def init(self):
        self.counter = 0

    # When no name is specified, the handler name is the method name.
    @Photon.handler
    def add(self, x: int) -> int:
        self.counter += x
        return self.counter

    # Or, we can specify a name for the handler.
    @Photon.handler("sub")
    def sub(self, x: int) -> int:
        return self.add(-x)



#%%
import csv
import sys
import duckdb
from pathlib import Path
from tqdm import tqdm

DATA_DIR = Path.joinpath(Path.home(), "data/github_archive_analytics")
USER_INFO_FILE = Path.joinpath(DATA_DIR, "user_info.csv")
WATCH_EVENTS_FILE = Path.joinpath(DATA_DIR, "watch_events.csv")
MISSING_USERS_FILE = Path.joinpath(DATA_DIR, "missing_users.csv")

def extract_missing_users():
    con = duckdb.connect()
    users = con.execute(f"SELECT DISTINCT stargazer_login, stargazer_url from '{WATCH_EVENTS_FILE}';").fetchall()
    fetched_users = con.execute(f"SELECT login from '{USER_INFO_FILE}';").fetchall()
    fetched_user_logins = set([user[0] for user in fetched_users])
    missed_users = []
    for user in tqdm(users):
        if user[0] not in fetched_user_logins:
            missed_users.append(user)
    with open(MISSING_USERS_FILE, "w+") as f:
        writer = csv.writer(f)
        writer.writerow(["user_login", "user_url"])
        writer.writerows(missed_users)
    con.close()

extract_missing_users()



#%%
import csv
import sys
import duckdb
from pathlib import Path
from tqdm import tqdm

DATA_DIR = Path.joinpath(Path.home(), "data/github_archive_analytics")
ANALYTICS_DIR = Path.joinpath(DATA_DIR, "analytics")
USER_INFO_FILE = Path.joinpath(DATA_DIR, "user_info.csv")
WATCH_EVENTS_FILE = Path.joinpath(DATA_DIR, "watch_events.csv")
MISSING_USERS_FILE = Path.joinpath(DATA_DIR, "missing_users.csv")

def extract_related_repos(repo:str):
    con = duckdb.connect()
    query = f"""
SELECT DISTINCT r2.repo_full_name, r1.stargazer_login AS common_stargazer, r1.created_at as create_at
FROM '{WATCH_EVENTS_FILE}' r1
JOIN '{WATCH_EVENTS_FILE}' r2 ON r1.stargazer_login = r2.stargazer_login
WHERE r1.repo_full_name = '{repo}' AND r2.repo_full_name <> '{repo}';
""".strip()
    repo_stargazers = con.execute(query).fetchall()
    with open(f"{ANALYTICS_DIR}/related_repos_{repo.replace('/', '-')}.csv", "w+") as f:
        writer = csv.writer(f)
        writer.writerow(["repo_full_name", "common_stargazer", "created_at"])
        writer.writerows(repo_stargazers)
    con.close()

def extract_related_watch_events(repo:str):
    con = duckdb.connect()
    related_repos_file = f"{ANALYTICS_DIR}/related_repos_{repo.replace('/', '-')}.csv"
    related_watch_events_file = f"{ANALYTICS_DIR}/related_watch_events_{repo.replace('/', '-')}.csv"
    query = f"""
SELECT * FROM '{WATCH_EVENTS_FILE}'
WHERE repo_full_name in (SELECT repo_full_name FROM '{related_repos_file}');
""".strip()
    related_watch_events = con.execute(query).fetchall()
    with open(related_watch_events_file, "w+") as f:
        writer = csv.writer(f)
        writer.writerow(["stargazer_login", "stargazer_url", "repo_full_name", "repo_url", "created_at"])
        writer.writerows(related_watch_events)
    con.close()

repo = "litanlitudan/skyagi"
# extract_related_repos(repo)
# extract_related_watch_events(repo)



from ghagent.util.conversion import df_to_textual_table
import pandas as pd
from textual.app import App
from textual.widgets import Header, Footer, DataTable, Label
from textual.widget import Widget
from textual.binding import Binding


DUMMY_ROWS = pd.DataFrame(data=[
    [4, "Joseph Schooling", "Singapore", 50.39],
    [2, "Michael Phelps", "United States", 51.14],
    [5, "Chad le Clos", "South Africa", 51.14],
    [6, "László Cseh", "Hungary", 51.14],
    [3, "Li Zhuhao", "China", 51.26],
    [8, "Mehdy Metella", "France", 51.58],
    [7, "Tom Shields", "United States", 51.73],
    [1, "Aleksandr Sadovnikov", "Russia", 51.84],
    [10, "Darren Burns", "Scotland", 51.84],
], columns=["lane", "swimmer", "country", "time"])

class DFTable(Widget):
    """
    Table with a pandas DataFrame as its data source
    """
    def __init__(self, data: pd.DataFrame, show_table_name: bool, table_name: str):
        super().__init__()
        self.data = data
        self.show_table_name = show_table_name
        self.table_name = table_name
    
    def compose(self):
        if self.show_table_name:
            yield Label(renderable=self.table_name, classes="table_name")
        
        data_table = DataTable(show_cursor=False)
        data_table = df_to_textual_table(self.data, data_table)
        yield data_table

class GHAgentApp(App):
    CSS_PATH = "tui.tcss"
    
    BINDINGS = [
        Binding(key="q", action="quit", description="Quit the app"),
    ]
    
    def __init__(self, relevant_related_repos, potential_stargazers):
        super().__init__()
        self.relevant_related_repos = relevant_related_repos
        self.potential_stargazers = potential_stargazers
    
    def compose(self):
        yield Header()
        
        yield DFTable(data=self.relevant_related_repos, show_table_name=True, table_name="Related Repos")
        yield DFTable(data=self.potential_stargazers, show_table_name=True, table_name="Potential Stargazers")
        
        yield Footer()

if __name__ == "__main__":
    app = GHAgentApp(relevant_related_repos=DUMMY_ROWS, potential_stargazers=DUMMY_ROWS)
    app.run()



import json
import os
from pathlib import Path
from typing import Any, Dict


def load_json(filepath: Path) -> Dict:
    if not Path(filepath).exists():
        return {}
    with open(filepath, "r") as file:
        try:
            json_obj = json.load(file)
            return json_obj
        except json.JSONDecodeError as e:
            if os.stat(filepath).st_size == 0:
                # Empty file
                return {}
            else:
                raise e

def load_json_value(filepath: Path, key: str, default_value: Any) -> Any:
    if not Path(filepath).exists():
        return default_value
    json_obj = load_json(filepath)
    if key not in json_obj:
        return default_value
    return json_obj[key]


def set_json_value(filepath: Path, key: str, value: Any) -> None:
    # key needs to follow python naming convention, such as trial_id
    json_obj = load_json(filepath)
    json_obj[key] = value
    with open(filepath, "w+") as json_file:
        json.dump(json_obj, json_file, sort_keys=True)
        json_file.flush()
